@@ -26,14 +26,17 @@ from calculate.core.setup_package import ChainProgressTemplate
 from calculate.lib.cl_template import templateFunction, SystemIni
 from calculate.lib.datavars import DataVars, Variable
 from calculate.lib.utils.colortext import (TextState, get_color_print)
-from calculate.lib.utils.common import CmdlineParams
+from calculate.lib.utils.common import CmdlineParams, getTupleVersion
 from calculate.lib.utils.portage import BinaryPackage, get_binary_file
 from calculate.lib.utils.files import (
     pathJoin, PercentProgress, getProgPath, process, STDOUT, removeDir,
     makeDirectory, writeFile, readLinesFile, chmod, chown, FilePermission,
     find, FindFileType, removeFileWithEmptyDirectory, check_rw,
+    tar_xz_directory,
     copyWithPath, countFiles, listDirectory, getRunCommands, isMount, readFile)
 from calculate.lib.utils.git import Git
+from calculate.builder.variables.action import Actions
+from hashlib import sha256
 from calculate.lib.utils.portage import (Layman, EmergeLog, EmergeLogNamedTask,
                                          InstalledPackageInfo, EbuildInfoError,
                                          EbuildInfo, ChrootEix, getRequires,
@@ -43,6 +46,8 @@ from calculate.lib.utils.portage import (Layman, EmergeLog, EmergeLogNamedTask,
                                          hide_packages, unhide_packages,
                                          LibraryProviders,
                                          PackageError, getInstalledAtom)
+from calculate.lib.utils.tools import ReverseKey
+from itertools import groupby
 from calculate.update.emerge_parser import (EmergeParser,
                                             EmergeError, EmergeCommand, Chroot,
                                             Linux32, CommandExecutor,
@@ -52,7 +57,7 @@ from calculate.update.update_tasks import EmergeMark
 from .build_storage import Build
 from calculate.update.update import Update
 from calculate.install.distr import (Distributive, IsoDistributive,
-                                     DistributiveError)
+                                     DistributiveError, ContainerDistributive)
 import shutil
 from functools import partial
 import os
@@ -81,6 +86,7 @@ class Builder(Update):
         Image = "builder_image"
         Profile = "builder_profile"
         UpdateMenu = "update_livemenu"
+        Container = "builder_container"
         All = (Prepare, Break, Update, Restore, Image, Profile)
 
     def init(self):
@@ -157,6 +163,107 @@ class Builder(Update):
         self.startTask(_("Pack ISO image"))
         self.addProgress()
 
+    def prepare_container_data(self, dn):
+        """
+        Подготовить мета данные rootfs.tar.xz
+        :param dn:
+        :return:
+        """
+        self.set_builder_action(Actions.ImageContainer)
+        root_path = path.relpath(dn, self.clVars.Get('cl_builder_path'))
+        self.applyTemplates(self.clVars.Get('cl_builder_target'),
+                            False, False, root_path)
+        self.endTask()
+        return True
+
+    def pack_container_data(self, dn, image):
+        """
+        Запаковать содержимое в dn к образу root.tar.xz
+        :param dn:
+        :param image:
+        :return:
+        """
+        for fn in listDirectory(dn, fullPath=True):
+            if path.isdir(fn):
+                fn_basename = path.basename(fn)
+                tar_xz_directory(fn,
+                                 path.join(image.basedirectory,
+                                           "%s.tar.xz") % fn_basename)
+            else:
+                fn_basename = path.basename(fn)
+                shutil.copy(fn, path.join(image, fn_basename))
+        return True
+
+    def create_digest_container(self, image):
+        """
+        Создать контрольную сумму для файлов контейнера
+        :param dn:
+        :return:
+        """
+        sumfn = "SHA256SUMS"
+        with writeFile(path.join(image.basedirectory, sumfn)) as f_sum:
+            for fn in listDirectory(image.basedirectory, fullPath=True):
+                if fn.endswith(sumfn):
+                    continue
+                kb = 1024
+                hasher = sha256()
+                with open(fn) as f:
+                    b = True
+                    while b:
+                        b = f.read(512 * kb)
+                        hasher.update(b)
+                digest = hasher.hexdigest()
+                f_sum.write("%s %s\n" % (digest, path.basename(fn)))
+        return True
+
+    def update_http_meta(self, dn):
+        """
+        Обновить индексный файл для lxc-download (meta/1.0/index-system)
+        :param dn:
+        :return:
+        """
+        data = []
+        re_subbuild = re.compile("\/[^-]+-[^-]+-(\d+)-[^-]+$", re.I)
+        for container_dn in listDirectory(dn, fullPath=True):
+            with ContainerDistributive(container_dn) as distro:
+                info = distro.get_information()
+                if not info['cl_profile_name']:
+                    continue
+                match_subbuild = re_subbuild.search(container_dn)
+                if match_subbuild:
+                    subbuild = int(match_subbuild.group(1))
+                else:
+                    subbuild = 0
+                data.append({
+                    'dist': info['os_linux_shortname'],
+                    'release': info['os_linux_ver'],
+                    'arch': info['os_arch_machine'],
+                    'variant': "default",
+                    'build': info['os_linux_build'],
+                    'fn': path.join("container", path.basename(container_dn)),
+                    'subbuild': subbuild
+                })
+
+        def sort_key(info):
+            return (ReverseKey(getTupleVersion(info['release'])),
+                    info['arch'],
+                    info['dist'], ReverseKey(info['build']),
+                    ReverseKey(info['subbuild']),
+                    ReverseKey(info['fn']))
+        data.sort(key=sort_key)
+        with writeFile(path.join(dn, 'meta/1.0/index-system')) as f:
+            for k, grps in groupby(data, lambda x: (x['dist'],
+                                                    x['release'],
+                                                    x['arch'],
+                                                    x['variant'])):
+                f.write("{dist};{release};{arch};"
+                        "{variant};{build};{fn}\n".format(**grps.next()))
+        return True
+
+    def remove_container_data(self, dn):
+        removeDir(dn)
+        return True
+
     def prepare_image(self, image):
         image.eventPrepareIso.connect(self.prepare_iso)
         return True
