@@ -108,19 +108,21 @@ class DataVarsBuilderLinux(linux.LinuxDataVars):
                     system.VariableOsArchMachineGentoo()]
 
     @classmethod
-    def StageInformation(cls, fn):
-        re_stage = re.compile("stage(\d)-(amd64|x86)-(\d{8}).tar.\w+")
-        m = re_stage.search(fn)
-        if m:
-            map_arch = {'amd64': 'x86_64', 'x86': 'i686'}
-            c = {'os_linux_build': m.group(3),
-                 'os_arch_machine': map_arch[m.group(2)],
-                 'os_linux_shortname': 'Gentoo',
-                 'os_linux_subname': "Stage%s" % m.group(1),
-                 'cl_profile_name': 'Gentoo profile',
-                 'os_linux_name': 'Gentoo'}
+    def StageInformation(cls, image, fn):
+        c = {}
+        if isinstance(image, distr.ContainerDistributive):
+            c = image.get_information()
         else:
-            c = {}
+            re_stage = re.compile("stage(\d)-(amd64|x86)-(\d{8}).tar.\w+")
+            m = re_stage.search(fn)
+            if m:
+                map_arch = {'amd64': 'x86_64', 'x86': 'i686'}
+                c = {'os_linux_build': m.group(3),
+                     'os_arch_machine': map_arch[m.group(2)],
+                     'os_linux_shortname': 'Gentoo',
+                     'os_linux_subname': "Stage%s" % m.group(1),
+                     'cl_profile_name': 'Gentoo profile',
+                     'os_linux_name': 'Gentoo'}
         obj = cls(cache=c)
         for vn in obj.allVars.keys():
             if vn not in obj.cache:
@@ -174,7 +176,7 @@ class VariableClBuilderLinuxDatavars(ReadonlyVariable):
     def get_data(self, image, image_fn=None, source=False):
         if image:
             if isinstance(image, distr.ArchiveDistributive):
-                return DataVarsBuilderLinux.StageInformation(image_fn)
+                return DataVarsBuilderLinux.StageInformation(image, image_fn)
             with image:
                 with ignore(DistributiveError):
                     distr_dn = image.getDirectory()
